"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ommx.v1.constraint_pb2
import ommx.v1.decision_variables_pb2
import ommx.v1.function_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Instance(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Description(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        DESCRIPTION_FIELD_NUMBER: builtins.int
        AUTHORS_FIELD_NUMBER: builtins.int
        CREATED_BY_FIELD_NUMBER: builtins.int
        name: builtins.str
        description: builtins.str
        created_by: builtins.str
        """The application or library name that created this message."""
        @property
        def authors(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            description: builtins.str | None = ...,
            authors: collections.abc.Iterable[builtins.str] | None = ...,
            created_by: builtins.str | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["_created_by", b"_created_by", "_description", b"_description", "_name", b"_name", "created_by", b"created_by", "description", b"description", "name", b"name"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["_created_by", b"_created_by", "_description", b"_description", "_name", b"_name", "authors", b"authors", "created_by", b"created_by", "description", b"description", "name", b"name"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing.Literal["_created_by", b"_created_by"]) -> typing.Literal["created_by"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing.Literal["_description", b"_description"]) -> typing.Literal["description"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing.Literal["_name", b"_name"]) -> typing.Literal["name"] | None: ...

    DESCRIPTION_FIELD_NUMBER: builtins.int
    DECISION_VARIABLES_FIELD_NUMBER: builtins.int
    OBJECTIVE_FIELD_NUMBER: builtins.int
    CONSTRAINTS_FIELD_NUMBER: builtins.int
    @property
    def description(self) -> global___Instance.Description: ...
    @property
    def decision_variables(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ommx.v1.decision_variables_pb2.DecisionVariable]:
        """Decision variables used in this instance

        - This must constain every decision variables used in the objective and constraints.
        - This can contains a decision variable that is not used in the objective or constraints.
        """

    @property
    def objective(self) -> ommx.v1.function_pb2.Function: ...
    @property
    def constraints(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ommx.v1.constraint_pb2.Constraint]:
        """Constraints of the optimization problem"""

    def __init__(
        self,
        *,
        description: global___Instance.Description | None = ...,
        decision_variables: collections.abc.Iterable[ommx.v1.decision_variables_pb2.DecisionVariable] | None = ...,
        objective: ommx.v1.function_pb2.Function | None = ...,
        constraints: collections.abc.Iterable[ommx.v1.constraint_pb2.Constraint] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["description", b"description", "objective", b"objective"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["constraints", b"constraints", "decision_variables", b"decision_variables", "description", b"description", "objective", b"objective"]) -> None: ...

global___Instance = Instance



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/quadratic.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ommx.v1 import linear_pb2 as ommx_dot_v1_dot_linear__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17ommx/v1/quadratic.proto\x12\x07ommx.v1\x1a\x14ommx/v1/linear.proto\"\x8a\x01\n\tQuadratic\x12\x12\n\x04rows\x18\x01 \x03(\x04R\x04rows\x12\x18\n\x07\x63olumns\x18\x02 \x03(\x04R\x07\x63olumns\x12\x16\n\x06values\x18\x03 \x03(\x01R\x06values\x12,\n\x06linear\x18\x04 \x01(\x0b\x32\x0f.ommx.v1.LinearH\x00R\x06linear\x88\x01\x01\x42\t\n\x07_linearBZ\n\x0b\x63om.ommx.v1B\x0eQuadraticProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.quadratic_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\016QuadraticProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_QUADRATIC']._serialized_start=59
  _globals['_QUADRATIC']._serialized_end=197
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/constraint.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ommx.v1 import function_pb2 as ommx_dot_v1_dot_function__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18ommx/v1/constraint.proto\x12\x07ommx.v1\x1a\x16ommx/v1/function.proto\"\x83\x03\n\nConstraint\x12\x0e\n\x02id\x18\x01 \x01(\x04R\x02id\x12\x38\n\x08\x65quality\x18\x02 \x01(\x0e\x32\x1c.ommx.v1.Constraint.EqualityR\x08\x65quality\x12-\n\x08\x66unction\x18\x03 \x01(\x0b\x32\x11.ommx.v1.FunctionR\x08\x66unction\x12\x46\n\x0b\x64\x65scription\x18\x04 \x01(\x0b\x32\x1f.ommx.v1.Constraint.DescriptionH\x00R\x0b\x64\x65scription\x88\x01\x01\x1a\x39\n\x0b\x44\x65scription\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n\x06\x66orall\x18\x02 \x03(\x03R\x06\x66orall\"i\n\x08\x45quality\x12\x18\n\x14\x45QUALITY_UNSPECIFIED\x10\x00\x12\x1a\n\x16\x45QUALITY_EQUAL_TO_ZERO\x10\x01\x12\'\n#EQUALITY_LESS_THAN_OR_EQUAL_TO_ZERO\x10\x02\x42\x0e\n\x0c_descriptionB[\n\x0b\x63om.ommx.v1B\x0f\x43onstraintProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.constraint_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\017ConstraintProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_CONSTRAINT']._serialized_start=62
  _globals['_CONSTRAINT']._serialized_end=449
  _globals['_CONSTRAINT_DESCRIPTION']._serialized_start=269
  _globals['_CONSTRAINT_DESCRIPTION']._serialized_end=326
  _globals['_CONSTRAINT_EQUALITY']._serialized_start=328
  _globals['_CONSTRAINT_EQUALITY']._serialized_end=433
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/linear.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14ommx/v1/linear.proto\x12\x07ommx.v1\"\x8a\x01\n\x06Linear\x12*\n\x05terms\x18\x01 \x03(\x0b\x32\x14.ommx.v1.Linear.TermR\x05terms\x12\x1a\n\x08\x63onstant\x18\x02 \x01(\x01R\x08\x63onstant\x1a\x38\n\x04Term\x12\x0e\n\x02id\x18\x01 \x01(\x04R\x02id\x12 \n\x0b\x63oefficient\x18\x02 \x01(\x01R\x0b\x63oefficientBW\n\x0b\x63om.ommx.v1B\x0bLinearProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.linear_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\013LinearProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_LINEAR']._serialized_start=34
  _globals['_LINEAR']._serialized_end=172
  _globals['_LINEAR_TERM']._serialized_start=116
  _globals['_LINEAR_TERM']._serialized_end=172
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Solution(google.protobuf.message.Message):
    """A solution obtained by the solver."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class EntriesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.float
        def __init__(
            self,
            *,
            key: builtins.int = ...,
            value: builtins.float = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    ENTRIES_FIELD_NUMBER: builtins.int
    @property
    def entries(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.float]: ...
    def __init__(
        self,
        *,
        entries: collections.abc.Mapping[builtins.int, builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["entries", b"entries"]) -> None: ...

global___Solution = Solution

@typing.final
class SolutionList(google.protobuf.message.Message):
    """List of solutions obtained by the solver.
    This message is for supporting solvers that return multiple solutions.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOLUTIONS_FIELD_NUMBER: builtins.int
    @property
    def solutions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Solution]: ...
    def __init__(
        self,
        *,
        solutions: collections.abc.Iterable[global___Solution] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["solutions", b"solutions"]) -> None: ...

global___SolutionList = SolutionList



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/function.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ommx.v1 import linear_pb2 as ommx_dot_v1_dot_linear__pb2
from ommx.v1 import polynomial_pb2 as ommx_dot_v1_dot_polynomial__pb2
from ommx.v1 import quadratic_pb2 as ommx_dot_v1_dot_quadratic__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16ommx/v1/function.proto\x12\x07ommx.v1\x1a\x14ommx/v1/linear.proto\x1a\x18ommx/v1/polynomial.proto\x1a\x17ommx/v1/quadratic.proto\"\xca\x01\n\x08\x46unction\x12\x1c\n\x08\x63onstant\x18\x01 \x01(\x01H\x00R\x08\x63onstant\x12)\n\x06linear\x18\x02 \x01(\x0b\x32\x0f.ommx.v1.LinearH\x00R\x06linear\x12\x32\n\tquadratic\x18\x03 \x01(\x0b\x32\x12.ommx.v1.QuadraticH\x00R\tquadratic\x12\x35\n\npolynomial\x18\x04 \x01(\x0b\x32\x13.ommx.v1.PolynomialH\x00R\npolynomialB\n\n\x08\x66unctionBY\n\x0b\x63om.ommx.v1B\rFunctionProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.function_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\rFunctionProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_FUNCTION']._serialized_start=109
  _globals['_FUNCTION']._serialized_end=311
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Bound(google.protobuf.message.Message):
    """Upper and lower bound of the decision variable."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOWER_FIELD_NUMBER: builtins.int
    UPPER_FIELD_NUMBER: builtins.int
    lower: builtins.float
    """Lower bound of the decision variable."""
    upper: builtins.float
    """Upper bound of the decision variable."""
    def __init__(
        self,
        *,
        lower: builtins.float = ...,
        upper: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["lower", b"lower", "upper", b"upper"]) -> None: ...

global___Bound = Bound

@typing.final
class DecisionVariable(google.protobuf.message.Message):
    """Decison variable which mathematical programming solver will optimize.
    It must have its kind, i.e. binary, integer, real or others and unique identifier of 64-bit integer.
    It may have its name and subscripts which are used to identify in modeling tools.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Kind:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _KindEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[DecisionVariable._Kind.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        KIND_UNSPECIFIED: DecisionVariable._Kind.ValueType  # 0
        KIND_BINARY: DecisionVariable._Kind.ValueType  # 1
        KIND_INTEGER: DecisionVariable._Kind.ValueType  # 2
        KIND_CONTINUOUS: DecisionVariable._Kind.ValueType  # 3
        KIND_SEMI_INTEGER: DecisionVariable._Kind.ValueType  # 4
        """Semi-integer decision variable is a decision variable that can take only integer values in the given range or zero."""
        KIND_SEMI_CONTINUOUS: DecisionVariable._Kind.ValueType  # 5
        """Semi-continuous decision variable is a decision variable that can take only continuous values in the given range or zero."""

    class Kind(_Kind, metaclass=_KindEnumTypeWrapper):
        """Kind of the decision variable"""

    KIND_UNSPECIFIED: DecisionVariable.Kind.ValueType  # 0
    KIND_BINARY: DecisionVariable.Kind.ValueType  # 1
    KIND_INTEGER: DecisionVariable.Kind.ValueType  # 2
    KIND_CONTINUOUS: DecisionVariable.Kind.ValueType  # 3
    KIND_SEMI_INTEGER: DecisionVariable.Kind.ValueType  # 4
    """Semi-integer decision variable is a decision variable that can take only integer values in the given range or zero."""
    KIND_SEMI_CONTINUOUS: DecisionVariable.Kind.ValueType  # 5
    """Semi-continuous decision variable is a decision variable that can take only continuous values in the given range or zero."""

    @typing.final
    class Description(google.protobuf.message.Message):
        """Human readable description of the decision variable."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        SUBSCRIPTS_FIELD_NUMBER: builtins.int
        name: builtins.str
        """Name of the decision variable."""
        @property
        def subscripts(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """The subscripts of a deicision variable which is defined as multi-dimensional array.
            Empty list means that the decision variable is scalar
            """

        def __init__(
            self,
            *,
            name: builtins.str = ...,
            subscripts: collections.abc.Iterable[builtins.int] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["name", b"name", "subscripts", b"subscripts"]) -> None: ...

    ID_FIELD_NUMBER: builtins.int
    KIND_FIELD_NUMBER: builtins.int
    BOUND_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    id: builtins.int
    """Decision variable ID.

    - IDs are not required to be sequential.
    """
    kind: global___DecisionVariable.Kind.ValueType
    """Kind of the decision variable"""
    @property
    def bound(self) -> global___Bound:
        """Bound of the decision variable
        If the bound is not specified, the decision variable is considered as unbounded.
        """

    @property
    def description(self) -> global___DecisionVariable.Description:
        """This is optional since the name and subscripts does not exist in general mathematical programming situation"""

    def __init__(
        self,
        *,
        id: builtins.int = ...,
        kind: global___DecisionVariable.Kind.ValueType = ...,
        bound: global___Bound | None = ...,
        description: global___DecisionVariable.Description | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_bound", b"_bound", "_description", b"_description", "bound", b"bound", "description", b"description"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_bound", b"_bound", "_description", b"_description", "bound", b"bound", "description", b"description", "id", b"id", "kind", b"kind"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_bound", b"_bound"]) -> typing.Literal["bound"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_description", b"_description"]) -> typing.Literal["description"] | None: ...

global___DecisionVariable = DecisionVariable



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/sparse_matrix.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bommx/v1/sparse_matrix.proto\x12\x07ommx.v1\"\x90\x01\n\x0cSparseMatrix\x12\x19\n\x08num_rows\x18\x01 \x01(\x04R\x07numRows\x12\x1f\n\x0bnum_columns\x18\x02 \x01(\x04R\nnumColumns\x12\x12\n\x04rows\x18\x03 \x03(\x04R\x04rows\x12\x18\n\x07\x63olumns\x18\x04 \x03(\x04R\x07\x63olumns\x12\x16\n\x06values\x18\x05 \x03(\x01R\x06valuesB]\n\x0b\x63om.ommx.v1B\x11SparseMatrixProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.sparse_matrix_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\021SparseMatrixProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_SPARSEMATRIX']._serialized_start=41
  _globals['_SPARSEMATRIX']._serialized_end=185
# @@protoc_insertion_point(module_scope)



import enum

import numpy as np

from ommx.v1.constraint_pb2 import Constraint
from ommx.v1.decision_variables_pb2 import DecisionVariable, Bound
from ommx.v1.function_pb2 import Function
from ommx.v1.instance_pb2 import Instance
from ommx.v1.linear_pb2 import Linear
from ommx.v1.solution_pb2 import Solution, SolutionList


class DataType(enum.Enum):
    INT = enum.auto()
    FLOAT = enum.auto()


class SingleFeasibleLPGenerator:
    INT_LOWER_BOUND = -100
    INT_UPPER_BOUND = 100
    FLOAT_LOWER_BOUND = -100.0
    FLOAT_UPPER_BOUND = 100.0

    def __init__(self, n: int, data_type: DataType):
        """
        The class generates a test instance as follows:

        Objective function: 0
        Constraints: A @ x = b    (A: regular matrix, b: constant vector)

        So the generated instance has a unique solution `x`.

        Args:
            n (int): The size of the matrix and the vectors.
            data_type (DataType): The data type of the matrix and the vectors.
        
        Raises:
            ValueError: If `n` is not a positive integer or `data_type` is not DataType.
        """
        if n <= 0:
            raise ValueError("`n` must be a positive integer.")
        if data_type not in DataType:
            raise ValueError("`data_type` must be DataType.")
        
        self._A = self._generate_random_reguler_matrix(n, data_type)
        self._x = self._generate_random_solution(n, data_type)
        self._b = self._A @ self._x
        self._data_type = data_type

        
    def _generate_random_reguler_matrix(
        self,
        n: int,
        data_type: DataType,
    ) -> np.ndarray:
        while True:
            if data_type == DataType.INT:
                matrix = np.random.randint(
                    low=self.INT_LOWER_BOUND,
                    high=self.INT_UPPER_BOUND,
                    size=(n, n),
                )
            else:
                matrix = np.random.rand(n, n)

            if np.linalg.det(matrix) != 0:
                return matrix

            
    def _generate_random_solution(
        self,
        n: int,
        data_type: DataType,
    ) -> np.ndarray:
        if data_type == DataType.INT:
            return np.random.randint(
                low=self.INT_LOWER_BOUND,
                high=self.INT_UPPER_BOUND,
                size=n,
            )
        else:
            return np.random.uniform(
                low=self.FLOAT_LOWER_BOUND,
                high=self.FLOAT_UPPER_BOUND,
                size=n
            )


    def get_v1_instance(self) -> bytes:
        """
        Get an instance of a linear programming problem with a unique solution.

        Examples:
            >>> from ommx.testing import DataType, SingleFeasibleLPGenerator
            >>> from ommx.v1.instance_pb2 import Instance
            >>> generator = SingleFeasibleLPGenerator(3, DataType.INT)
            >>> ommx_instance_byte = generator.get_v1_instance()
            >>> ommx_instance = Instance().ParseFromString(ommx_instance_byte)
        """
        # define decision variables
        if self._data_type == DataType.INT:
            decision_variables = [
                DecisionVariable(
                    id=i,
                    kind=DecisionVariable.Kind.KIND_INTEGER,
                    bound=Bound(
                        lower=self.INT_LOWER_BOUND,
                        upper=self.INT_UPPER_BOUND,
                    )
                )
                for i in range(len(self._x))
            ]
        else:
            decision_variables = [
                DecisionVariable(
                    id=i,
                    kind=DecisionVariable.Kind.KIND_CONTINUOUS,
                    bound=Bound(
                        lower= self.FLOAT_LOWER_BOUND,
                        upper= self.FLOAT_UPPER_BOUND,
                    )
                )
                for i in range(len(self._x))
            ]

        # define constraints
        constraints = []
        for i in range(len(self._b)):
            linear = Linear(
                terms=[
                    Linear.Term(id=j, coefficient=value)
                    for j, value in enumerate(self._A[i])
                ],
                constant=-self._b[i],
            )
            
            constraint = Constraint(
                id=i,
                equality=Constraint.Equality.EQUALITY_EQUAL_TO_ZERO,
                function=Function(constant=-self._b[i], linear=linear),
            )
            constraints.append(constraint)

        return Instance(
            description=Instance.Description(name="LPTest"),
            decision_variables=decision_variables,
            objective=Function(constant=0),
            constraints=constraints,
        ).SerializeToString()


    def get_v1_solution(self) -> bytes:
        """
        Get the solution of the generated instance.

        Examples:
            >>> from ommx.testing import DataType, SingleFeasibleLPGenerator
            >>> from ommx.v1.solution_pb2 import SolutionList
            >>> generator = SingleFeasibleLPGenerator(3, DataType.INT)
            >>> ommx_solution_byte = generator.get_v1_solution()
            >>> ommx_solution = SolutionList().ParseFromString(ommx_solution_byte)
        """
        solution = Solution(
            entries={i: value for i, value in enumerate(self._x)}
        )
        return SolutionList(solutions=[solution]).SerializeToString()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Linear(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Term(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ID_FIELD_NUMBER: builtins.int
        COEFFICIENT_FIELD_NUMBER: builtins.int
        id: builtins.int
        coefficient: builtins.float
        def __init__(
            self,
            *,
            id: builtins.int = ...,
            coefficient: builtins.float = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["coefficient", b"coefficient", "id", b"id"]) -> None: ...

    TERMS_FIELD_NUMBER: builtins.int
    CONSTANT_FIELD_NUMBER: builtins.int
    constant: builtins.float
    @property
    def terms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Linear.Term]: ...
    def __init__(
        self,
        *,
        terms: collections.abc.Iterable[global___Linear.Term] | None = ...,
        constant: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["constant", b"constant", "terms", b"terms"]) -> None: ...

global___Linear = Linear



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/solution.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16ommx/v1/solution.proto\x12\x07ommx.v1\"\x80\x01\n\x08Solution\x12\x38\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x1e.ommx.v1.Solution.EntriesEntryR\x07\x65ntries\x1a:\n\x0c\x45ntriesEntry\x12\x10\n\x03key\x18\x01 \x01(\x04R\x03key\x12\x14\n\x05value\x18\x02 \x01(\x01R\x05value:\x02\x38\x01\"?\n\x0cSolutionList\x12/\n\tsolutions\x18\x01 \x03(\x0b\x32\x11.ommx.v1.SolutionR\tsolutionsBY\n\x0b\x63om.ommx.v1B\rSolutionProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.solution_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\rSolutionProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_SOLUTION_ENTRIESENTRY']._loaded_options = None
  _globals['_SOLUTION_ENTRIESENTRY']._serialized_options = b'8\001'
  _globals['_SOLUTION']._serialized_start=36
  _globals['_SOLUTION']._serialized_end=164
  _globals['_SOLUTION_ENTRIESENTRY']._serialized_start=106
  _globals['_SOLUTION_ENTRIESENTRY']._serialized_end=164
  _globals['_SOLUTIONLIST']._serialized_start=166
  _globals['_SOLUTIONLIST']._serialized_end=229
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/decision_variables.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n ommx/v1/decision_variables.proto\x12\x07ommx.v1\"3\n\x05\x42ound\x12\x14\n\x05lower\x18\x01 \x01(\x01R\x05lower\x12\x14\n\x05upper\x18\x02 \x01(\x01R\x05upper\"\xb4\x03\n\x10\x44\x65\x63isionVariable\x12\x0e\n\x02id\x18\x01 \x01(\x04R\x02id\x12\x32\n\x04kind\x18\x02 \x01(\x0e\x32\x1e.ommx.v1.DecisionVariable.KindR\x04kind\x12)\n\x05\x62ound\x18\x03 \x01(\x0b\x32\x0e.ommx.v1.BoundH\x00R\x05\x62ound\x88\x01\x01\x12L\n\x0b\x64\x65scription\x18\x04 \x01(\x0b\x32%.ommx.v1.DecisionVariable.DescriptionH\x01R\x0b\x64\x65scription\x88\x01\x01\x1a\x41\n\x0b\x44\x65scription\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1e\n\nsubscripts\x18\x02 \x03(\x04R\nsubscripts\"\x85\x01\n\x04Kind\x12\x14\n\x10KIND_UNSPECIFIED\x10\x00\x12\x0f\n\x0bKIND_BINARY\x10\x01\x12\x10\n\x0cKIND_INTEGER\x10\x02\x12\x13\n\x0fKIND_CONTINUOUS\x10\x03\x12\x15\n\x11KIND_SEMI_INTEGER\x10\x04\x12\x18\n\x14KIND_SEMI_CONTINUOUS\x10\x05\x42\x08\n\x06_boundB\x0e\n\x0c_descriptionBb\n\x0b\x63om.ommx.v1B\x16\x44\x65\x63isionVariablesProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.decision_variables_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\026DecisionVariablesProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_BOUND']._serialized_start=45
  _globals['_BOUND']._serialized_end=96
  _globals['_DECISIONVARIABLE']._serialized_start=99
  _globals['_DECISIONVARIABLE']._serialized_end=535
  _globals['_DECISIONVARIABLE_DESCRIPTION']._serialized_start=308
  _globals['_DECISIONVARIABLE_DESCRIPTION']._serialized_end=373
  _globals['_DECISIONVARIABLE_KIND']._serialized_start=376
  _globals['_DECISIONVARIABLE_KIND']._serialized_end=509
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import ommx.v1.function_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Constraint(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Equality:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _EqualityEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Constraint._Equality.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        EQUALITY_UNSPECIFIED: Constraint._Equality.ValueType  # 0
        EQUALITY_EQUAL_TO_ZERO: Constraint._Equality.ValueType  # 1
        EQUALITY_LESS_THAN_OR_EQUAL_TO_ZERO: Constraint._Equality.ValueType  # 2

    class Equality(_Equality, metaclass=_EqualityEnumTypeWrapper): ...
    EQUALITY_UNSPECIFIED: Constraint.Equality.ValueType  # 0
    EQUALITY_EQUAL_TO_ZERO: Constraint.Equality.ValueType  # 1
    EQUALITY_LESS_THAN_OR_EQUAL_TO_ZERO: Constraint.Equality.ValueType  # 2

    @typing.final
    class Description(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        FORALL_FIELD_NUMBER: builtins.int
        name: builtins.str
        @property
        def forall(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            forall: collections.abc.Iterable[builtins.int] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["forall", b"forall", "name", b"name"]) -> None: ...

    ID_FIELD_NUMBER: builtins.int
    EQUALITY_FIELD_NUMBER: builtins.int
    FUNCTION_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    id: builtins.int
    """Constraint ID

    - Constraint IDs are managed separately from decision variable IDs.
      We can use the same ID for both. For example, we have a decision variable `x` with decision variable ID `1``
      and constraint `x == 0` with constraint ID `1`.
    - IDs are not required to be sequential.
    - IDs must be unique with other types of constraints.
    """
    equality: global___Constraint.Equality.ValueType
    @property
    def function(self) -> ommx.v1.function_pb2.Function: ...
    @property
    def description(self) -> global___Constraint.Description: ...
    def __init__(
        self,
        *,
        id: builtins.int = ...,
        equality: global___Constraint.Equality.ValueType = ...,
        function: ommx.v1.function_pb2.Function | None = ...,
        description: global___Constraint.Description | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_description", b"_description", "description", b"description", "function", b"function"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_description", b"_description", "description", b"description", "equality", b"equality", "function", b"function", "id", b"id"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["_description", b"_description"]) -> typing.Literal["description"] | None: ...

global___Constraint = Constraint



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ommx/v1/instance.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ommx.v1 import constraint_pb2 as ommx_dot_v1_dot_constraint__pb2
from ommx.v1 import decision_variables_pb2 as ommx_dot_v1_dot_decision__variables__pb2
from ommx.v1 import function_pb2 as ommx_dot_v1_dot_function__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16ommx/v1/instance.proto\x12\x07ommx.v1\x1a\x18ommx/v1/constraint.proto\x1a ommx/v1/decision_variables.proto\x1a\x16ommx/v1/function.proto\"\xb3\x03\n\x08Instance\x12?\n\x0b\x64\x65scription\x18\x01 \x01(\x0b\x32\x1d.ommx.v1.Instance.DescriptionR\x0b\x64\x65scription\x12H\n\x12\x64\x65\x63ision_variables\x18\x02 \x03(\x0b\x32\x19.ommx.v1.DecisionVariableR\x11\x64\x65\x63isionVariables\x12/\n\tobjective\x18\x03 \x01(\x0b\x32\x11.ommx.v1.FunctionR\tobjective\x12\x35\n\x0b\x63onstraints\x18\x04 \x03(\x0b\x32\x13.ommx.v1.ConstraintR\x0b\x63onstraints\x1a\xb3\x01\n\x0b\x44\x65scription\x12\x17\n\x04name\x18\x01 \x01(\tH\x00R\x04name\x88\x01\x01\x12%\n\x0b\x64\x65scription\x18\x02 \x01(\tH\x01R\x0b\x64\x65scription\x88\x01\x01\x12\x18\n\x07\x61uthors\x18\x03 \x03(\tR\x07\x61uthors\x12\"\n\ncreated_by\x18\x04 \x01(\tH\x02R\tcreatedBy\x88\x01\x01\x42\x07\n\x05_nameB\x0e\n\x0c_descriptionB\r\n\x0b_created_byBY\n\x0b\x63om.ommx.v1B\rInstanceProtoP\x01\xa2\x02\x03OXX\xaa\x02\x07Ommx.V1\xca\x02\x07Ommx\\V1\xe2\x02\x13Ommx\\V1\\GPBMetadata\xea\x02\x08Ommx::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ommx.v1.instance_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013com.ommx.v1B\rInstanceProtoP\001\242\002\003OXX\252\002\007Ommx.V1\312\002\007Ommx\\V1\342\002\023Ommx\\V1\\GPBMetadata\352\002\010Ommx::V1'
  _globals['_INSTANCE']._serialized_start=120
  _globals['_INSTANCE']._serialized_end=555
  _globals['_INSTANCE_DESCRIPTION']._serialized_start=376
  _globals['_INSTANCE_DESCRIPTION']._serialized_end=555
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Monomial(google.protobuf.message.Message):
    """A monomial in a multivariate polynomial."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IDS_FIELD_NUMBER: builtins.int
    COEFFICIENT_FIELD_NUMBER: builtins.int
    coefficient: builtins.float
    @property
    def ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        ids: collections.abc.Iterable[builtins.int] | None = ...,
        coefficient: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["coefficient", b"coefficient", "ids", b"ids"]) -> None: ...

global___Monomial = Monomial

@typing.final
class Polynomial(google.protobuf.message.Message):
    """Multi­variate polynomial"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TERMS_FIELD_NUMBER: builtins.int
    @property
    def terms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Monomial]: ...
    def __init__(
        self,
        *,
        terms: collections.abc.Iterable[global___Monomial] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["terms", b"terms"]) -> None: ...

global___Polynomial = Polynomial



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import ommx.v1.linear_pb2
import ommx.v1.polynomial_pb2
import ommx.v1.quadratic_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Function(google.protobuf.message.Message):
    """Real-valued multivariate function used for objective function and constraints."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONSTANT_FIELD_NUMBER: builtins.int
    LINEAR_FIELD_NUMBER: builtins.int
    QUADRATIC_FIELD_NUMBER: builtins.int
    POLYNOMIAL_FIELD_NUMBER: builtins.int
    constant: builtins.float
    """Constant function like `f(x_1, x_2) = 2`"""
    @property
    def linear(self) -> ommx.v1.linear_pb2.Linear:
        """Linear function like `f(x_1, x_2) = 2 x_1 + 3 x_2`"""

    @property
    def quadratic(self) -> ommx.v1.quadratic_pb2.Quadratic:
        """Quadratic function like `f(x_1, x_2) = 4 x_1 x_2 + 5 x_2`"""

    @property
    def polynomial(self) -> ommx.v1.polynomial_pb2.Polynomial:
        """Polynomial like `f(x_1, x_2) = 4 x_1^2 + 5 x_2^3 + 6 x_1 x_2^2 + 7 x_2^2 + 8 x_1 x_2 + 9 x_1 + 10 x_2 + 11`"""

    def __init__(
        self,
        *,
        constant: builtins.float = ...,
        linear: ommx.v1.linear_pb2.Linear | None = ...,
        quadratic: ommx.v1.quadratic_pb2.Quadratic | None = ...,
        polynomial: ommx.v1.polynomial_pb2.Polynomial | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["constant", b"constant", "function", b"function", "linear", b"linear", "polynomial", b"polynomial", "quadratic", b"quadratic"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["constant", b"constant", "function", b"function", "linear", b"linear", "polynomial", b"polynomial", "quadratic", b"quadratic"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["function", b"function"]) -> typing.Literal["constant", "linear", "quadratic", "polynomial"] | None: ...

global___Function = Function



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ommx.v1.linear_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Quadratic(google.protobuf.message.Message):
    """Quadratic function as a COO-style sparse matrix and linear sparse vector.

    COOrdinate format, also known as triplet format, is a way to represent sparse matrices as a list of non-zero elements.
    It consists of three lists: the row indices, the column indices, and the values of the non-zero elements.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    VALUES_FIELD_NUMBER: builtins.int
    LINEAR_FIELD_NUMBER: builtins.int
    @property
    def rows(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def columns(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def linear(self) -> ommx.v1.linear_pb2.Linear: ...
    def __init__(
        self,
        *,
        rows: collections.abc.Iterable[builtins.int] | None = ...,
        columns: collections.abc.Iterable[builtins.int] | None = ...,
        values: collections.abc.Iterable[builtins.float] | None = ...,
        linear: ommx.v1.linear_pb2.Linear | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_linear", b"_linear", "linear", b"linear"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_linear", b"_linear", "columns", b"columns", "linear", b"linear", "rows", b"rows", "values", b"values"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["_linear", b"_linear"]) -> typing.Literal["linear"] | None: ...

global___Quadratic = Quadratic
